Package: ggbio
Version: 1.60.0
Title: Visualization tools for genomic data
Description: The ggbio package extends and specializes the grammar of
        graphics for biological data. The graphics are designed to
        answer common scientific questions, in particular those often
        asked of high throughput genomics data. All core Bioconductor
        data structures are supported, where appropriate. The package
        supports detailed views of particular genomic regions, as well
        as genome-wide overviews. Supported overviews include ideograms
        and grand linear views. High-level plots include sequence
        fragment length, edge-linked interval to data view, mismatch
        pileup, and several splicing summaries.
Authors@R: c(person("Tengfei", "Yin", role=c("aut"),
                 email="yintengfei@gmail.com"),
          person("Michael", "Lawrence", role=c("aut", "ths", "cre"),
                 email="lawremi@gmail.com"),
          person("Dianne", "Cook", role=c("aut", "ths")),
          person("Sanchit", "Saini", role="ctb"),
          person("Johannes", "Rainer", role="ctb"))
Depends: methods, BiocGenerics, ggplot2 (>= 1.0.0)
Imports: grid, grDevices, graphics, stats, utils, gridExtra, scales,
        reshape2, gtable, Hmisc, biovizBase (>= 1.29.2), Biobase,
        S4Vectors (>= 0.13.13), IRanges (>= 2.11.16), Seqinfo,
        GenomeInfoDb (>= 1.45.5), GenomicRanges (>= 1.61.1),
        SummarizedExperiment (>= 1.39.1), Biostrings (>= 2.77.2),
        Rsamtools (>= 2.25.1), GenomicAlignments (>= 1.45.1), BSgenome
        (>= 1.77.1), VariantAnnotation (>= 1.55.1), rtracklayer (>=
        1.69.1), GenomicFeatures (>= 1.61.4), OrganismDbi, ensembldb
        (>= 2.33.1), AnnotationDbi, AnnotationFilter, rlang
VignetteBuilder: knitr
Suggests: vsn, BSgenome.Hsapiens.UCSC.hg19, Homo.sapiens,
        TxDb.Hsapiens.UCSC.hg19.knownGene,
        TxDb.Mmusculus.UCSC.mm9.knownGene, knitr, BiocStyle, testthat,
        EnsDb.Hsapiens.v75, tinytex
URL: https://lawremi.github.io/ggbio/
BugReports: https://github.com/lawremi/ggbio/issues
License: Artistic-2.0
LazyLoad: Yes
Collate: AllClasses.R AllGenerics.R Cache-class.R GGbio-class.R
        Grob-class.R ideogram.R Tracked-class.R Plot-class.R
        ggplot-method.R theme.R Tracks-class.R geom_chevron-method.R
        geom_alignment-method.R geom_arch-method.R geom_arrow-method.R
        geom_arrowrect-method.R geom_rect-method.R
        geom_segment-method.R geom_bar-method.R layout_circle-method.R
        layout_karyogram-method.R layout_linear-method.R
        stat_aggregate-method.R stat_coverage-method.R
        stat_identity-method.R stat_mismatch-method.R
        stat_stepping-method.R stat_gene-method.R stat_table-method.R
        stat_bin-method.R stat_slice-method.R stat_reduce-method.R
        coord_genome-method.R autoplot-method.R hack.R
        plotGrandLinear.R plotRangesLinkedToData.R
        plotFragLength-method.R plotSpliceSum-method.R rescale-method.R
        scales.R utils.R zzz.R
biocViews: Infrastructure, Visualization
git_url: https://git.bioconductor.org/packages/ggbio
git_branch: RELEASE_3_23
git_last_commit: 41e5a61
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:24:33 UTC; biocbuild
Author: Tengfei Yin [aut],
  Michael Lawrence [aut, ths, cre],
  Dianne Cook [aut, ths],
  Sanchit Saini [ctb],
  Johannes Rainer [ctb]
Maintainer: Michael Lawrence <lawremi@gmail.com>
